@@ -50,7 +50,7 @@ CppWriter::CppWriter()
     // allows you to specify where the vector variable should be in your code,
     // and "%ITEMCLASS%", if needed, where the class of the item is declared.
     VECTOR_METHOD_APPEND = QLatin1String("%VARNAME%.push_back(add_object);"); // for std::vector
-    VECTOR_METHOD_REMOVE = QLatin1String("int i, size = %VARNAME%.size();\nfor (i = 0; i < size; ++i) {\n\t%ITEMCLASS% item = %VARNAME%.at(i);\n\tif(item == remove_object) {\n\t\tvector<%ITEMCLASS%>::iterator it = %VARNAME%.begin() + i;\n\t\t%VARNAME%.erase(it);\n\t\treturn;\n\t}\n }"); // for std::vector
+    VECTOR_METHOD_REMOVE = QString(QLatin1String("int i, size = %VARNAME%.size();\nfor (i = 0; i < size; ++i) {\n\t%ITEMCLASS% item = %VARNAME%.at(i);\n\tif(item == remove_object) {\n\t\t%1<%ITEMCLASS%>::iterator it = %VARNAME%.begin() + i;\n\t\t%VARNAME%.erase(it);\n\t\treturn;\n\t}\n }")).arg(policyExt()->getVectorClassName()); // for std::vector
     VECTOR_METHOD_INIT.clear(); // nothing to be done
     /*
         VECTOR_METHOD_APPEND = QLatin1String("%VARNAME%.append(&add_object);"); // Qt lib implementation
@@ -123,7 +123,7 @@ void CppWriter::writeClass(UMLClassifier *c)
     // Determine whether the implementation file is required.
     // (It is not required if the class is an enumeration.)
     bool need_impl = true;
-    if (c->baseType() == UMLObject::ot_Enum) {
+    if (c->baseType() == UMLObject::ot_Enum || c->isInterface()) {
         need_impl = false;
     }
     if (need_impl) {
@@ -346,7 +346,7 @@ void CppWriter::writeClassDecl(UMLClassifier *c, QTextStream &cpp)
     }
 
     writeBlankLine(cpp);
-    cpp << "#include <" << policyExt()->getStringClassNameInclude() << ">" << m_endl;
+    cpp << "#include " << policyExt()->getStringClassNameInclude() << m_endl;
     if (c->hasVectorFields())
     {
         cpp << "#include " << policyExt()->getVectorClassNameInclude() << m_endl;
@@ -449,32 +449,44 @@ void CppWriter::writeClassDecl(UMLClassifier *c, QTextStream &cpp)
     // write out field and operations decl grouped by visibility
     //
 
+    QString s;
+    QTextStream tmp(&s);
+
     // PUBLIC attribs/methods
-    cpp << "public:" << m_endl << m_endl; // print visibility decl.
-    writeDataTypes(c, Uml::Visibility::Public, cpp);
+    writeDataTypes(c, Uml::Visibility::Public, tmp);
     // for public: constructors are first ops we print out
     if (!c->isInterface())
-        writeConstructorDecls(cpp);
-    writeHeaderFieldDecl(c, Uml::Visibility::Public, cpp);
-    writeHeaderAccessorMethodDecl(c, Uml::Visibility::Public, cpp);
-    writeOperations(c, true, Uml::Visibility::Public, cpp);
+        writeConstructorDecls(tmp);
+    writeHeaderFieldDecl(c, Uml::Visibility::Public, tmp);
+    writeHeaderAccessorMethodDecl(c, Uml::Visibility::Public, tmp);
+    writeOperations(c, true, Uml::Visibility::Public, tmp);
+    s = s.trimmed();
+    if (!s.isEmpty()) {
+        cpp << "public:" << m_endl << indent() << s << m_endl << m_endl;
+    }
 
+    s.clear();
     // PROTECTED attribs/methods
-    //
-    cpp << "protected" << ":" << m_endl << m_endl; // print visibility decl.
-    writeDataTypes(c, Uml::Visibility::Protected, cpp);
-    writeHeaderFieldDecl(c, Uml::Visibility::Protected, cpp);
-    writeHeaderAccessorMethodDecl(c, Uml::Visibility::Protected, cpp);
-    writeOperations(c, true, Uml::Visibility::Protected, cpp);
+    writeDataTypes(c, Uml::Visibility::Protected, tmp);
+    writeHeaderFieldDecl(c, Uml::Visibility::Protected, tmp);
+    writeHeaderAccessorMethodDecl(c, Uml::Visibility::Protected, tmp);
+    writeOperations(c, true, Uml::Visibility::Protected, tmp);
+    s = s.trimmed();
+    if (!s.isEmpty()) {
+        cpp << "protected:" << m_endl << indent() << s << m_endl << m_endl;
+    }
 
+    s.clear();
     // PRIVATE attribs/methods
-    //
-    cpp << "private" << ":" << m_endl << m_endl; // print visibility decl.
-    writeDataTypes(c, Uml::Visibility::Private, cpp);
-    writeHeaderFieldDecl(c, Uml::Visibility::Private, cpp);
-    writeHeaderAccessorMethodDecl(c, Uml::Visibility::Private, cpp);
-    writeOperations(c, true, Uml::Visibility::Private, cpp);
-    writeInitAttributeDecl(c, cpp); // this is always private, used by constructors to initialize class
+    writeDataTypes(c, Uml::Visibility::Private, tmp);
+    writeHeaderFieldDecl(c, Uml::Visibility::Private, tmp);
+    writeHeaderAccessorMethodDecl(c, Uml::Visibility::Private, tmp);
+    writeOperations(c, true, Uml::Visibility::Private, tmp);
+    writeInitAttributeDecl(c, tmp); // this is always private, used by constructors to initialize class
+    s = s.trimmed();
+    if (!s.isEmpty()) {
+        cpp << "private:" << m_endl << indent() << s << m_endl;
+    }
 
     // end of class header
     cpp << m_endl << "};" << m_endl;
@@ -561,16 +573,8 @@ void CppWriter::writeHeaderAttributeAccessorMethods (UMLClassifier *c, Uml::Visi
     else
         list = c->getAttributeList(visibility);
 
-    // switch to public
-    if (visibility != Uml::Visibility::Public)
-        stream << "public:" << m_endl << m_endl;
-
     // write accessor methods for attribs we found
     writeAttributeMethods(list, visibility, true, false, policyExt()->getAccessorsAreInline(), stream);
-
-    // switch back to previous vis.
-    if (visibility != Uml::Visibility::Public)
-        stream << Uml::Visibility::toString(visibility) << ":" << m_endl << m_endl;
 }
 
 /**
@@ -601,14 +605,9 @@ void CppWriter::writeAttributeMethods(UMLAttributeList attribs,
 
     foreach (UMLAttribute* at, attribs) {
         QString varName = getAttributeVariableName(at);
-        QString methodBaseName = cleanName(at->name());
+        QString methodBaseName = getAttributeMethodBaseName(cleanName(at->name()));
 
-        // force capitalizing the field name, this is silly,
-        // from what I can tell, this IS the default behavior for
-        // cleanName. I dunno why it is not working -b.t.
         methodBaseName = methodBaseName.trimmed();
-        methodBaseName.replace(0, 1, methodBaseName.at(0).toUpper());
-
         writeSingleAttributeAccessorMethods(at->getTypeName(), varName,
                                             methodBaseName, at->doc(), Uml::Changeability::Changeable, isHeaderMethod,
                                             at->isStatic(), writeMethodBody, stream);
@@ -902,7 +901,7 @@ void CppWriter::writeVectorAttributeAccessorMethods (
     stream << indnt << returnVarName << " ";
     if (!isHeaderMethod)
         stream << className_ << "::";
-    stream << "get" << fldName << "List ()";
+    stream << "get" << fldName << "List()";
     if (writeMethodBody) {
         stream << indnt << " {" << m_endl;
         m_indentLevel++;
@@ -932,7 +931,6 @@ void CppWriter::writeSingleAttributeAccessorMethods(
         return;
 
     QString className = fixTypeName(fieldClassName);
-    QString fldName = Codegen_Utils::capitalizeFirstLetter(fieldName);
     QString indnt = indent();
     QString varName = QLatin1String("new_var");
     QString fullVarName = varName;
@@ -944,6 +942,9 @@ void CppWriter::writeSingleAttributeAccessorMethods(
         className = className.left(i);
         isArrayType = true;
     }
+    QString fldName = fieldName;
+    if (policyExt()->getAccessorMethodsStartWithUpperCase())
+        fldName = Codegen_Utils::capitalizeFirstLetter(fieldName);
 
     // set method
     if (change == Uml::Changeability::Changeable) {
@@ -951,12 +952,12 @@ void CppWriter::writeSingleAttributeAccessorMethods(
         stream << indnt << "void ";
         if (!isHeaderMethod)
             stream << className_ << "::";
-        stream << "set" << fldName << " (" << className << " " << fullVarName << ")";
+        stream << "set" << fldName << "(" << className << " " << fullVarName << ")";
 
         if (writeMethodBody) {
-            stream << indnt << " {" << m_endl;
+            stream << m_endl << indnt << "{" << m_endl;
             m_indentLevel++;
-            stream << indent() << indnt;
+            stream << indent();
             m_indentLevel--;
             if (isStatic)
                 stream << className_ << "::";
@@ -969,7 +970,7 @@ void CppWriter::writeSingleAttributeAccessorMethods(
             stream << ";" << m_endl;
     }
 
-    if (i)
+    if (i > -1)
         className += QLatin1String("*");
 
     // get method
@@ -977,10 +978,13 @@ void CppWriter::writeSingleAttributeAccessorMethods(
     stream << indnt << className << " ";
     if (!isHeaderMethod)
         stream << className_ << "::";
-    stream << "get" << fldName << " ()";
+    if (policyExt()->getGetterWithGetPrefix())
+        stream << "get" << fldName << "()";
+    else
+        stream << fieldName << "()";
 
     if (writeMethodBody) {
-        stream << indnt << " {" << m_endl;
+        stream << m_endl << indnt << "{" << m_endl;
         m_indentLevel++;
         stream << indent() << "return ";
         m_indentLevel--;
@@ -1013,10 +1017,10 @@ void CppWriter::writeConstructorDecls(QTextStream &stream)
         return;
 
     writeDocumentation(QString(), QLatin1String("Empty Constructor"), QString(), stream);
-    stream << indent() << className_ << " ();" << m_endl;
+    stream << indent() << className_ << "();" << m_endl;
     writeDocumentation(QString(), QLatin1String("Empty Destructor"), QString(), stream);
     stream << indent();
-    stream << "virtual ~" << className_ << " ();" << m_endl;
+    stream << "virtual ~" << className_ << "();" << m_endl;
     writeBlankLine(stream);
 }
 
@@ -1027,7 +1031,7 @@ void CppWriter::writeInitAttributeDecl(UMLClassifier * c, QTextStream &stream)
 {
     if (UMLApp::app()->commonPolicy()->getAutoGenerateConstructors() &&
         c->hasAttributes())
-        stream << indent() << "void initAttributes () ;" << m_endl;
+        stream << indent() << "void initAttributes();" << m_endl;
 }
 
 /**
@@ -1042,7 +1046,8 @@ void CppWriter::writeInitAttributeMethod(UMLClassifier * c, QTextStream &stream)
 
     QString indnt = indent();
 
-    stream << indnt << "void " << className_ << "::" << "initAttributes () {" << m_endl;
+    stream << indnt << "void " << className_ << "::" << "initAttributes()" << m_endl
+           << "{" << m_endl;
 
     m_indentLevel++;
     // first, initiation of fields derived from attributes
@@ -1103,14 +1108,19 @@ void CppWriter::writeConstructorMethods(UMLClassifier * c, QTextStream &stream)
 
     // empty constructor
     QString indnt = indent();
-    stream << indnt << className_ << "::" << className_ << " () {" << m_endl;
+    stream << indent() << className_ << "::" << className_ << "()" << m_endl
+           << indent() << "{" << m_endl;
+    m_indentLevel++;
     if (c->hasAttributes())
-        stream << indnt << indnt << "initAttributes();" << m_endl;
-    stream << indnt << "}" << m_endl;
+        stream << indent() << "initAttributes();" << m_endl;
+    m_indentLevel--;
+    stream << indent() << "}" << m_endl;
     writeBlankLine(stream);
 
     // empty destructor
-    stream << indent() << className_ << "::~" << className_ << " () { }" << m_endl;
+    stream << indent() << className_ << "::~" << className_ << "()" << m_endl
+           << indent() << "{" << m_endl
+           << indent() << "}" << m_endl;
     writeBlankLine(stream);
 }
 
@@ -1123,6 +1133,7 @@ void CppWriter::writeConstructorMethods(UMLClassifier * c, QTextStream &stream)
 void CppWriter::writeDataTypes(UMLClassifier *c, Uml::Visibility::Enum permitScope, QTextStream &stream)
 {
     foreach (UMLObject* o, c->containedObjects()) {
+        uIgnoreZeroPointer(o);
         if (o->visibility() != permitScope)
             continue;
         if (!o->isUMLDatatype())
@@ -1226,7 +1237,7 @@ void CppWriter::writeOperations(UMLClassifier *c, UMLOperationList &oplist, bool
         if (!isHeaderMethod)
             str += className_ + QLatin1String("::");
 
-        str += cleanName(op->name()) + QLatin1String(" (");
+        str += cleanName(op->name()) + QLatin1String("(");
 
         // generate parameters
         uint j = 0;
@@ -1255,8 +1266,14 @@ void CppWriter::writeOperations(UMLClassifier *c, UMLOperationList &oplist, bool
         if (op->getConst())
             str += QLatin1String(" const");
 
+        if (op->getOverride())
+            str += QLatin1String(" override");
+
+        if (isHeaderMethod && op->isAbstract()) {
+            str += QLatin1String(" = 0;");
+        }
         // method body : only gets IF it is not in a header
-        if (isHeaderMethod && !policyExt()->getOperationsAreInline()) {
+        else if (isHeaderMethod && !policyExt()->getOperationsAreInline()) {
             str += QLatin1Char(';'); // terminate now
         }
         else {
@@ -1367,10 +1384,24 @@ void CppWriter::printTextAsSeparateLinesWithIndent(const QString &text, const QS
  */
 QString CppWriter::getAttributeVariableName(UMLAttribute *at)
 {
-    QString fieldName = cleanName(at->name());
+    QString fieldName;
+    CodeGenPolicyExt *pe = UMLApp::app()->policyExt();
+    CPPCodeGenerationPolicy * policy = dynamic_cast<CPPCodeGenerationPolicy*>(pe);
+    if (policy && !policy->getClassMemberPrefix().isEmpty())
+        fieldName = policy->getClassMemberPrefix() + cleanName(at->name());
+    else
+        fieldName = cleanName(at->name());
     return fieldName;
 }
 
+QString CppWriter::getAttributeMethodBaseName(const QString &fieldName)
+{
+    QString fldName = fieldName;
+    if (policyExt()->getRemovePrefixFromAccessorMethods())
+        fldName.replace(QRegExp(QLatin1String("^[a-zA-Z]_")), QLatin1String(""));
+    return fldName;
+}
+
 /**
  * Add C++ primitives as datatypes.
  * @return   the list of default datatypes
