@@ -21,6 +21,7 @@
 #include "objectwidget.h"
 #include "uml.h"
 #include "umldoc.h"
+#include "umlview.h"
 #include "umlobject.h"
 #include "umlscene.h"
 #include "widget_factory.h"
@@ -137,11 +138,13 @@ bool parseSequenceLine(const QString &s, QString &sequence, QString &package, QS
 
     /**
      * Qtcreator/gdb
+     * @verbatim
      *  6	Driver::ParseHelper::ParseHelper	driver.cpp	299	0x634c44
      * 31   g_main_context_dispatch /usr/lib64/libglib-2.0.so.0     0x7fffefe16316
      * ignoring
      * ... <more>                                                   0x7ffff41152d9
      * 13  ??                                                       0x7ffff41152d9
+     * @endverbatim
      */
     else if (cols[cols.size()-1].startsWith(QLatin1String("0x"))) {
         if (cols[0] == QLatin1String("...") || cols[1] == QLatin1String("??"))
@@ -317,7 +320,7 @@ bool importSequences(const QStringList &lines, UMLScene *scene, const QString &f
  */
 bool importGraph(const QStringList &lines, UMLScene *scene, const QString &fileName)
 {
-    if (scene->type() == Uml::DiagramType::Sequence)
+    if (scene->isSequenceDiagram())
         return importSequences(lines, scene);
     else if (scene->type() != Uml::DiagramType::Class)
         return false;
@@ -480,7 +483,7 @@ bool importGraph(const QMimeData* mimeData, UMLScene *scene)
     UMLDoc *doc = UMLApp::app()->document();
     doc->beginPaste();
     bool result = false;
-    if (scene->type() == Uml::DiagramType::Sequence)
+    if (scene->isSequenceDiagram())
         result = importSequences(lines, scene);
     else
         result = importGraph(lines, scene);
@@ -512,4 +515,24 @@ bool importGraph(const QString &fileName, UMLScene *scene)
     return importGraph(lines, scene, fileName);
 }
 
+/**
+ * Check if name for a diagram is unique
+ *
+ * @param type type of diagram to check (set to undefined if to check against all diagrams)
+ * @param name name of diagram to check
+ * @return true - name is unique
+ * @return false - name is not unique
+ */
+bool isUniqueDiagramName(Uml::DiagramType::Enum type, QString &name)
+{
+    bool found = false;
+    foreach (UMLView *view, UMLApp::app()->document()->viewIterator()) {
+        if (type == Uml::DiagramType::Undefined || view->umlScene()->type() == type) {
+            if (view->umlScene()->name() == name)
+                found = true;
+        }
+    }
+    return !found;
+}
+
 }  // end namespace Diagram_Utils
