@@ -287,7 +287,7 @@ void WidgetBasePopupMenu::insertSingleSelectionMenu(WidgetBase* widget)
         insertSubMenuNew(type);
         insertSubMenuColor(widget->useFillColor());
         if (widget->umlScene() &&
-            widget->umlScene()->type() == Uml::DiagramType::Sequence) {
+            widget->umlScene()->isSequenceDiagram()) {
             addSeparator();
             MenuType tabUp = mt_Up;
             insert(mt_Up, Icon_Utils::SmallIcon(Icon_Utils::it_Arrow_Up), i18n("Move Up"));
@@ -338,15 +338,22 @@ void WidgetBasePopupMenu::insertSingleSelectionMenu(WidgetBase* widget)
     case WidgetBase::wt_State:
         {
             StateWidget* pState = static_cast< StateWidget *>(widget);
-            if (pState->stateType() == StateWidget::Normal) {
-                // FIXME: why not using wt_state
-                insertSubMenuNew(WidgetBase::wt_Activity);
-            } else {
-                insertSubMenuNew(type);
+            switch (pState->stateType()) {
+            case StateWidget::Combined:
+                insert(mt_EditCombinedState);
+                break;
+            default:
+                break;
             }
+
+            insertSubMenuNew(type);
             insertSubMenuColor(widget->useFillColor());
             insertStdItems(false, type);
             switch (pState->stateType()) {
+            case StateWidget::Combined:
+                insert(mt_Change_Font);
+                insert(mt_Properties);
+                break;
             case StateWidget::Normal:
                 insert(mt_Rename, i18n("Change State Name..."));
                 insert(mt_Change_Font);
@@ -354,10 +361,7 @@ void WidgetBasePopupMenu::insertSingleSelectionMenu(WidgetBase* widget)
                 break;
             case StateWidget::Fork:
             case StateWidget::Join:
-                if (pState->drawVertical())
-                    insert(mt_Flip, i18n("Flip Horizontal"));
-                else
-                    insert(mt_Flip, i18n("Flip Vertical"));
+                insert(pState->drawVertical() ? mt_FlipHorizontal : mt_FlipVertical);
                 break;
             default:
                 break;
@@ -369,14 +373,8 @@ void WidgetBasePopupMenu::insertSingleSelectionMenu(WidgetBase* widget)
         insertSubMenuNew(type);
         {
             ForkJoinWidget *pForkJoin = static_cast<ForkJoinWidget*>(widget);
-            if (pForkJoin->orientation() == Qt::Vertical) {
-                insert(mt_Flip, i18n("Flip Horizontal"));
-            }
-            else {
-                insert(mt_Flip, i18n("Flip Vertical"));
-            }
-            m_actions[mt_Fill_Color] = addAction(Icon_Utils::SmallIcon(Icon_Utils::it_Color_Fill),
-                                                 i18n("Fill Color..."));
+            insert(pForkJoin->orientation() == Qt::Vertical ? mt_FlipHorizontal : mt_FlipVertical);
+            insert(mt_Fill_Color);
         }
         break;
 
@@ -601,8 +599,8 @@ void WidgetBasePopupMenu::insertSubMenuAlign()
 void WidgetBasePopupMenu::insertSubMenuColor(bool fc)
 {
     KMenu* color = newMenu(i18nc("color menu", "Color"), this);
-    insert(mt_Line_Color, color, Icon_Utils::SmallIcon(Icon_Utils::it_Color_Line), i18n("Line Color..."));
-    insert(mt_Fill_Color, color, Icon_Utils::SmallIcon(Icon_Utils::it_Color_Fill), i18n("Fill Color..."));
+    insert(mt_Line_Color, color);
+    insert(mt_Fill_Color, color);
     insert(mt_Use_Fill_Color, color, i18n("Use Fill Color"), CHECKABLE);
     setActionChecked(mt_Use_Fill_Color, fc);
     addMenu(color);
@@ -619,15 +617,40 @@ void WidgetBasePopupMenu::insertSubMenuNew(WidgetBase::WidgetType type, KMenu *m
         menu = makeNewMenu();
 
     switch (type) {
+        case WidgetBase::wt_Activity:
+            insert(mt_Initial_Activity, menu);
+            insert(mt_Activity, menu);
+            insert(mt_End_Activity, menu);
+            insert(mt_Final_Activity, menu);
+            insert(mt_Branch, menu);
+            insert(mt_Fork, menu);
+            insert(mt_Invoke_Activity, menu);
+            insert(mt_Param_Activity, menu);
+            insert(mt_Activity_Transition, menu);
+            insert(mt_Exception, menu);
+            insert(mt_PrePostCondition, menu);
+            insert(mt_Send_Signal, menu);
+            insert(mt_Accept_Signal, menu);
+            insert(mt_Accept_Time_Event, menu);
+            insert(mt_Region, menu);
+            insert(mt_Pin, menu);
+            insert(mt_Object_Node, menu);
+            break;
         case WidgetBase::wt_Actor:
         case WidgetBase::wt_UseCase:
             insert(mt_Actor, menu);
             insert(mt_UseCase, menu);
             break;
         case WidgetBase::wt_Component:
+            insert(mt_Subsystem, menu);
             insert(mt_Component, menu);
-            if (Settings::optionState().generalState.uml2)
+            if (Settings::optionState().generalState.uml2) {
                 insert(mt_Port, menu);
+                insert(mt_InterfaceProvided, menu);
+                insert(mt_InterfaceRequired, menu);
+            } else {
+                insert(mt_InterfaceComponent, menu);
+            }
             insert(mt_Artifact, menu);
             break;
         case WidgetBase::wt_Class:
@@ -657,6 +680,22 @@ void WidgetBasePopupMenu::insertSubMenuNew(WidgetBase::WidgetType type, KMenu *m
         case WidgetBase::wt_Enum:
             insert(mt_EnumLiteral, menu);
             break;
+        case WidgetBase::wt_Port:
+            insert(mt_InterfaceProvided, menu);
+            insert(mt_InterfaceRequired, menu);
+            break;
+        case WidgetBase::wt_State:
+            insert(mt_State, menu);
+            insert(mt_End_State, menu);
+            insert(mt_StateTransition, menu);
+            insert(mt_Junction, menu);
+            insert(mt_DeepHistory, menu);
+            insert(mt_ShallowHistory, menu);
+            insert(mt_Choice, menu);
+            insert(mt_StateFork, menu);
+            insert(mt_StateJoin, menu);
+            insert(mt_CombinedState, menu);
+            break;
         default:
             break;
     }
