@@ -436,12 +436,19 @@ QPen AssociationLine::pen() const
  */
 void AssociationLine::setPen(const QPen &pen)
 {
-    if (m_startSymbol)
+    if (m_startSymbol) {
         m_startSymbol->setPen(pen);
-    if (m_subsetSymbol)
+        // update brush fill color
+        m_startSymbol->setBrush(brush());
+    }
+    if (m_subsetSymbol) {
         m_subsetSymbol->setPen(pen);
-    if (m_endSymbol)
+        m_subsetSymbol->setBrush(brush());
+    }
+    if (m_endSymbol) {
         m_endSymbol->setPen(pen);
+        m_endSymbol->setBrush(brush());
+    }
 
     prepareGeometryChange();
     m_pen = pen;
@@ -512,9 +519,10 @@ void AssociationLine::calculateInitialEndPoints()
 void AssociationLine::reconstructSymbols()
 {
     switch( m_associationWidget->associationType() ) {
+        case Uml::AssociationType::Exception:
+            setLayout(Uml::LayoutType::Polyline);
         case Uml::AssociationType::State:
         case Uml::AssociationType::Activity:
-        case Uml::AssociationType::Exception:
         case Uml::AssociationType::UniAssociation:
         case Uml::AssociationType::Dependency:
             setStartSymbol(Symbol::None);
@@ -692,13 +700,13 @@ void AssociationLine::alignSymbols()
 
     QList<QPolygonF> polygons = path().toSubpathPolygons();
 
-    if (m_startSymbol) {
+    if (m_startSymbol && polygons.size() > 0) {
         QPolygonF firstLine = polygons.first();
         QLineF segment(firstLine.at(1), firstLine.at(0));
         m_startSymbol->alignTo(segment);
     }
 
-    if (m_endSymbol) {
+    if (m_endSymbol && polygons.size() > 0) {
         QPolygonF lastLine = polygons.last();
         int maxIndex = lastLine.size();
         QLineF segment(lastLine.at(maxIndex-2), lastLine.at(maxIndex-1));
