@@ -12,6 +12,7 @@
 #include "statewidget.h"
 
 // app includes
+#include "cmds/cmdcreatediagram.h"
 #include "debug_utils.h"
 #include "dialog_utils.h"
 #include "docwindow.h"
@@ -27,6 +28,7 @@
 #include <KLocalizedString>
 
 // qt includes
+#include <QtGlobal>
 #include <QPointer>
 
 /**
@@ -39,9 +41,8 @@
 StateWidget::StateWidget(UMLScene * scene, StateType stateType, Uml::ID::Type id)
   : UMLWidget(scene, WidgetBase::wt_State, id)
 {
-    m_stateType = stateType;
+    setStateType(stateType);
     m_drawVertical = true;
-    setAspectRatioMode();
     m_Text = QLatin1String("State");
     QSizeF size = minimumSize();
     setSize(size.width(), size.height());
@@ -181,6 +182,39 @@ void StateWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *optio
             painter->drawPolygon(polygon);
         }
         break;
+    case StateWidget::Combined:
+        {
+            const QFontMetrics &fm = getFontMetrics(FT_NORMAL);
+            const int fontHeight = fm.lineSpacing();
+            setPenFromSettings(painter);
+            QPainterPath path;
+            path.addRoundedRect(rect(), STATE_MARGIN, STATE_MARGIN);
+            if (useFillColor())
+                painter->fillPath(path, UMLWidget::fillColor());
+            painter->drawPath(path);
+            painter->drawLine(QPointF(0, fontHeight), QPointF(w, fontHeight));
+            painter->setPen(textColor());
+            QFont font = UMLWidget::font();
+            font.setBold(false);
+            painter->setFont(font);
+            painter->drawText(STATE_MARGIN, 0, w - STATE_MARGIN * 2, fontHeight,
+                              Qt::AlignCenter, name());
+            if (!m_linkedDiagram) {
+                m_size = QSizeF(fm.width(name()) + STATE_MARGIN * 2, fm.lineSpacing() + STATE_MARGIN);
+            } else {
+                m_sceneRect = m_linkedDiagram->sceneRect().adjusted(-1,-1, 1, 1);
+                m_clientRect = rect().adjusted(STATE_MARGIN, fontHeight + STATE_MARGIN, - STATE_MARGIN, -STATE_MARGIN);
+                if (Tracer::instance()->isEnabled(QLatin1String(metaObject()->className()))) {
+                    painter->setPen(Qt::magenta);
+                    painter->drawRect(m_clientRect);
+                }
+                m_linkedDiagram->render(painter, m_clientRect, m_sceneRect);
+                m_size = QSizeF(qMax<qreal>(fm.width(m_linkedDiagram->name()), m_sceneRect.width()) + STATE_MARGIN * 2, fm.lineSpacing() + STATE_MARGIN + m_sceneRect.height());
+                setSize(m_size);
+            }
+            setPenFromSettings(painter);
+        }
+        break;
     default:
         uWarning() << "Unknown state type: " << stateTypeStr();
         break;
@@ -244,6 +278,8 @@ QSizeF StateWidget::minimumSize() const
             width = 25;
             height = 25;
             break;
+        case StateWidget::Combined:
+            return m_size;
         default:
             break;
     }
@@ -271,6 +307,8 @@ QSizeF StateWidget::maximumSize()
                 return QSizeF(fontHeight + 10, fontHeight + 10);
             }
             break;
+        case StateWidget::Combined:
+            return m_size;
         default:
             break;
     }
@@ -300,6 +338,56 @@ void StateWidget::setAspectRatioMode()
     }
 }
 
+void StateWidget::contextMenuEvent(QGraphicsSceneContextMenuEvent* event)
+{
+#ifdef ENABLE_COMBINED_STATE_DIRECT_EDIT
+    if (m_stateType == Combined)
+        DiagramProxyWidget::contextMenuEvent(event);
+    if (event->isAccepted())
+#endif
+        UMLWidget::contextMenuEvent(event);
+}
+
+void StateWidget::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
+{
+#ifdef ENABLE_COMBINED_STATE_DIRECT_EDIT
+    if (m_stateType == Combined)
+        DiagramProxyWidget::mouseDoubleClickEvent(event);
+    if (event->isAccepted())
+#endif
+        UMLWidget::mouseDoubleClickEvent(event);
+}
+
+void StateWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
+{
+#ifdef ENABLE_COMBINED_STATE_DIRECT_EDIT
+    if (m_stateType == Combined)
+        DiagramProxyWidget::mousePressEvent(event);
+    if (event->isAccepted())
+#endif
+        UMLWidget::mousePressEvent(event);
+}
+
+void StateWidget::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
+{
+#ifdef ENABLE_COMBINED_STATE_DIRECT_EDIT
+    if (m_stateType == Combined)
+        DiagramProxyWidget::mouseMoveEvent(event);
+    if (event->isAccepted())
+#endif
+        UMLWidget::mouseMoveEvent(event);
+}
+
+void StateWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
+{
+#ifdef ENABLE_COMBINED_STATE_DIRECT_EDIT
+    if (m_stateType == Combined)
+        DiagramProxyWidget::mouseReleaseEvent(event);
+    if (event->isAccepted())
+#endif
+        UMLWidget::mouseReleaseEvent(event);
+}
+
 /**
  * Returns the type of state.
  * @return StateType
@@ -324,6 +412,10 @@ void StateWidget::setStateType(StateType stateType)
 {
     m_stateType = stateType;
     setAspectRatioMode();
+    if (stateType == Combined) {
+        setAutoResize(false);
+        setResizable(false);
+    }
 }
 
 /**
@@ -399,16 +491,19 @@ void StateWidget::setDrawVertical(bool to)
 /**
  * Show a properties dialog for a StateWidget.
  */
-void StateWidget::showPropertiesDialog()
+bool StateWidget::showPropertiesDialog()
 {
+    bool result = false;
     UMLApp::app()->docWindow()->updateDocumentation(false);
 
     QPointer<StateDialog> dialog = new StateDialog(m_scene->activeView(), this);
     if (dialog->exec() && dialog->getChangesMade()) {
         UMLApp::app()->docWindow()->showDocumentation(this, true);
         UMLApp::app()->document()->setModified(true);
+        result = true;
     }
     delete dialog;
+    return result;
 }
 
 /**
@@ -446,7 +541,7 @@ bool StateWidget::loadFromXMI1(QDomElement & qElement)
     m_Text = qElement.attribute(QLatin1String("statename"));
     m_Doc = qElement.attribute(QLatin1String("documentation"));
     QString type = qElement.attribute(QLatin1String("statetype"), QLatin1String("1"));
-    m_stateType = (StateType)type.toInt();
+    setStateType((StateType)type.toInt());
     setAspectRatioMode();
     QString drawVertical = qElement.attribute(QLatin1String("drawvertical"), QLatin1String("1"));
     m_drawVertical = (bool)drawVertical.toInt();
@@ -481,9 +576,7 @@ void StateWidget::slotMenuSelection(QAction* action)
     switch(sel) {
     case ListPopupMenu::mt_Rename:
         nameNew = name();
-        ok = Dialog_Utils::askName(i18n("Enter State Name"),
-                                   i18n("Enter the name of the new state:"),
-                                   nameNew);
+        ok = Dialog_Utils::askNewName(WidgetBase::WidgetType::wt_State, nameNew);
         if (ok && nameNew.length() > 0) {
             setName(nameNew);
         }
@@ -503,8 +596,30 @@ void StateWidget::slotMenuSelection(QAction* action)
         }
         break;
 
-    case ListPopupMenu::mt_Flip:
-        setDrawVertical(!m_drawVertical);
+    case ListPopupMenu::mt_FlipHorizontal:
+        setDrawVertical(false);
+        break;
+    case ListPopupMenu::mt_FlipVertical:
+        setDrawVertical(true);
+        break;
+
+    case ListPopupMenu::mt_CombinedState:
+        {
+            QString diagramName = m_doc->createDiagramName(Uml::DiagramType::State);
+            Uml::CmdCreateDiagram* d = new Uml::CmdCreateDiagram(m_doc, Uml::DiagramType::State, diagramName);
+            UMLApp::app()->executeCommand(d);
+            setDiagramLink(d->view()->umlScene()->ID());
+            setStateType(Combined);
+        }
+        break;
+
+    case ListPopupMenu::mt_EditCombinedState:
+        if (!m_linkedDiagram) {
+            uError() << "no diagram id defined at widget '" << Uml::ID::toString(id()) << "'";
+            break;
+        }
+        m_linkedDiagram->setWidgetLink(this);
+        UMLApp::app()->document()->changeCurrentView(m_diagramLinkId);
         break;
 
     default:
