@@ -11,24 +11,36 @@
 #include "umlwidget.h"
 
 // local includes
+#include "artifact.h"
+#include "artifactwidget.h"
+#include "activitywidget.h"
 #include "actor.h"
 #include "actorwidget.h"
 #include "associationwidget.h"
 #include "classifier.h"
 #include "classpropertiesdialog.h"
 #include "cmds.h"
+#include "component.h"
+#include "componentwidget.h"
 #include "debug_utils.h"
 #include "dialog_utils.h"
 #include "docwindow.h"
 #include "floatingtextwidget.h"
+#include "forkjoinwidget.h"
+#include "interfacewidget.h"
 #include "notewidget.h"
 #include "messagewidget.h"
 #include "object_factory.h"
 #include "idchangelog.h"
 #include "menus/listpopupmenu.h"
+#include "objectnodewidget.h"
+#include "pinwidget.h"
 #include "port.h"
 #include "portwidget.h"
+#include "regionwidget.h"
+#include "signalwidget.h"
 #include "settingsdialog.h"
+#include "statewidget.h"
 #include "uml.h"
 #include "umldoc.h"
 #include "umllistview.h"
@@ -71,6 +83,7 @@ const int UMLWidget::resizeMarkerLineCount = 3;
  */
 UMLWidget::UMLWidget(UMLScene * scene, WidgetType type, UMLObject * o)
   : WidgetBase(scene, type)
+  , DiagramProxyWidget(this)
 {
     init();
     m_umlObject = o;
@@ -91,6 +104,7 @@ UMLWidget::UMLWidget(UMLScene * scene, WidgetType type, UMLObject * o)
  */
 UMLWidget::UMLWidget(UMLScene *scene, WidgetType type, Uml::ID::Type id)
   : WidgetBase(scene, type)
+  , DiagramProxyWidget(this)
 {
     init();
     if (id == Uml::ID::None)
@@ -116,6 +130,7 @@ UMLWidget& UMLWidget::operator=(const UMLWidget & other)
         return *this;
 
     WidgetBase::operator=(other);
+    DiagramProxyWidget::operator=(other);
 
     // assign members loaded/saved
     m_useFillColor = other.m_useFillColor;
@@ -589,7 +604,9 @@ void UMLWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
         if ((m_inMoveArea && wasPositionChanged()) ||
                 (m_inResizeArea && wasSizeChanged())) {
             umlDoc()->setModified(true);
+            umlScene()->invalidate();
         }
+        umlScene()->resizeSceneToItems();
 
         UMLApp::app()->document()->writeToStatusBar(m_oldStatusBarMsg);
     }
@@ -844,6 +861,49 @@ void UMLWidget::slotMenuSelection(QAction *trigger)
         break;
     }
 
+    case ListPopupMenu::mt_Artifact: {
+        UMLArtifact *a = new UMLArtifact();
+        ArtifactWidget *widget = new ArtifactWidget(umlScene(), a);
+        addConnectedWidget(widget, Uml::AssociationType::Association);
+        break;
+    }
+
+    case ListPopupMenu::mt_Component: {
+        UMLComponent *c = new UMLComponent();
+        ComponentWidget *widget = new ComponentWidget(umlScene(), c);
+        addConnectedWidget(widget, Uml::AssociationType::Association, SetupSize);
+        break;
+    }
+
+    case ListPopupMenu::mt_Interface: {
+        UMLPackage* component = umlObject()->asUMLPackage();
+        QString name = Model_Utils::uniqObjectName(UMLObject::ot_Interface, component);
+        if (Dialog_Utils::askNewName(WidgetBase::wt_Interface, name)) {
+            UMLClassifier *c = new UMLClassifier();
+            c->setBaseType(UMLObject::ot_Interface);
+            ClassifierWidget *widget = new ClassifierWidget(umlScene(), c);
+            addConnectedWidget(widget, Uml::AssociationType::Association);
+        }
+        break;
+    }
+
+    case ListPopupMenu::mt_InterfaceComponent:
+    case ListPopupMenu::mt_InterfaceProvided: {
+        UMLObject *o = Object_Factory::createUMLObject(UMLObject::ot_Interface);
+        InterfaceWidget *w = new InterfaceWidget(umlScene(), o->asUMLClassifier());
+        w->setDrawAsCircle(true);
+        addConnectedWidget(w, Uml::AssociationType::Association, SetupSize);
+        break;
+    }
+
+    case ListPopupMenu::mt_InterfaceRequired: {
+        UMLObject *o = Object_Factory::createUMLObject(UMLObject::ot_Interface);
+        InterfaceWidget *w = new InterfaceWidget(umlScene(), o->asUMLClassifier());
+        w->setDrawAsCircle(true);
+        addConnectedWidget(w, Uml::AssociationType::Association, SetupSize | SwitchDirection);
+        break;
+    }
+
     case ListPopupMenu::mt_Note: {
         NoteWidget *widget = new NoteWidget(umlScene());
         addConnectedWidget(widget, Uml::AssociationType::Anchor);
@@ -854,11 +914,14 @@ void UMLWidget::slotMenuSelection(QAction *trigger)
         // TODO: merge with ToolbarStateOneWidget::setWidget()
         UMLPackage* component = umlObject()->asUMLPackage();
         QString name = Model_Utils::uniqObjectName(UMLObject::ot_Port, component);
-        if (Dialog_Utils::askName(i18n("Enter Port Name"), i18n("Enter the port"), name)) {
+        if (Dialog_Utils::askNewName(WidgetBase::wt_Port, name)) {
             UMLPort *port = Object_Factory::createUMLObject(UMLObject::ot_Port, name, component)->asUMLPort();
             UMLWidget *umlWidget = Widget_Factory::createWidget(umlScene(), port);
             umlWidget->setParentItem(this);
-            umlScene()->setupNewWidget(umlWidget);
+            QPointF p = mapFromScene(umlScene()->pos());
+            umlWidget->setPos(p);
+            umlScene()->setupNewWidget(umlWidget, false);
+
         }
         break;
     }
@@ -878,6 +941,87 @@ void UMLWidget::slotMenuSelection(QAction *trigger)
     case ListPopupMenu::mt_MessageLost:
         break;
 
+    // activity diagrams
+    case ListPopupMenu::mt_Accept_Signal:
+        addConnectedWidget(new SignalWidget(umlScene(), SignalWidget::Accept), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Accept_Time_Event:
+        addConnectedWidget(new SignalWidget(umlScene(), SignalWidget::Time), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Activity:
+        addConnectedWidget(new ActivityWidget(umlScene(), ActivityWidget::Normal), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Activity_Transition:
+        addConnectedWidget(new ActivityWidget(umlScene(), ActivityWidget::Final), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Branch:
+        addConnectedWidget(new ActivityWidget(umlScene(), ActivityWidget::Branch), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Exception:
+        umlScene()->triggerToolbarButton(WorkToolBar::tbb_Exception);
+        break;
+    case ListPopupMenu::mt_Final_Activity:
+        addConnectedWidget(new ActivityWidget(umlScene(), ActivityWidget::Final), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Fork:
+        addConnectedWidget(new ForkJoinWidget(umlScene()), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_End_Activity:
+        addConnectedWidget(new ActivityWidget(umlScene(), ActivityWidget::End), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Initial_Activity:
+        addConnectedWidget(new ActivityWidget(umlScene(), ActivityWidget::Initial), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Invoke_Activity:
+        addConnectedWidget(new ActivityWidget(umlScene(), ActivityWidget::Invok), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Object_Node:
+        addConnectedWidget(new ObjectNodeWidget(umlScene(), ObjectNodeWidget::Data), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Pin:
+        umlScene()->triggerToolbarButton(WorkToolBar::tbb_Pin);
+        break;
+    case ListPopupMenu::mt_Param_Activity:
+        addConnectedWidget(new ActivityWidget(umlScene(), ActivityWidget::Param), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_PrePostCondition:
+        addConnectedWidget(new NoteWidget(umlScene(), NoteWidget::Normal), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Region:
+        addConnectedWidget(new RegionWidget(umlScene()), Uml::AssociationType::Activity, NoOption);
+        break;
+    case ListPopupMenu::mt_Send_Signal:
+        addConnectedWidget(new SignalWidget(umlScene(), SignalWidget::Send), Uml::AssociationType::Activity, NoOption);
+        break;
+
+  // state diagrams
+    case ListPopupMenu::mt_Choice:
+        addConnectedWidget(new StateWidget(umlScene(), StateWidget::Choice), Uml::AssociationType::State, NoOption);
+        break;
+    case ListPopupMenu::mt_DeepHistory:
+        addConnectedWidget(new StateWidget(umlScene(), StateWidget::DeepHistory), Uml::AssociationType::State, NoOption);
+        break;
+    case ListPopupMenu::mt_End_State:
+        addConnectedWidget(new StateWidget(umlScene(), StateWidget::End), Uml::AssociationType::State, NoOption);
+        break;
+    case ListPopupMenu::mt_Junction:
+        addConnectedWidget(new StateWidget(umlScene(), StateWidget::Junction), Uml::AssociationType::State, NoOption);
+        break;
+    case ListPopupMenu::mt_ShallowHistory:
+        addConnectedWidget(new StateWidget(umlScene(), StateWidget::ShallowHistory), Uml::AssociationType::State, NoOption);
+        break;
+    case ListPopupMenu::mt_State:
+        addConnectedWidget(new StateWidget(umlScene(), StateWidget::Normal), Uml::AssociationType::State, NoOption);
+        break;
+    case ListPopupMenu::mt_StateFork:
+        addConnectedWidget(new StateWidget(umlScene(), StateWidget::Fork), Uml::AssociationType::State, NoOption);
+        break;
+    case ListPopupMenu::mt_StateJoin:
+        addConnectedWidget(new StateWidget(umlScene(), StateWidget::Join), Uml::AssociationType::State, NoOption);
+        break;
+    case ListPopupMenu::mt_StateTransition:
+        umlScene()->triggerToolbarButton(WorkToolBar::tbb_State_Transition);
+        break;
     default:
         WidgetBase::slotMenuSelection(trigger);
         break;
@@ -1073,8 +1217,9 @@ void UMLWidget::setFillColorCmd(const QColor &color)
  * @param ChangeLog
  * @return  true for success
  */
-bool UMLWidget::activate(IDChangeLog* /*ChangeLog  = 0 */)
+bool UMLWidget::activate(IDChangeLog* changeLog)
 {
+    DiagramProxyWidget::activate(changeLog);
     if (widgetHasUMLObject(baseType()) && m_umlObject == 0) {
         m_umlObject = m_doc->findObjectById(m_nId);
         if (m_umlObject == 0) {
@@ -1091,7 +1236,7 @@ bool UMLWidget::activate(IDChangeLog* /*ChangeLog  = 0 */)
         QPointF point = m_scene->getPastePoint();
         int x = point.x() + this->x();
         int y = point.y() + this->y();
-        if (m_scene->type() == Uml::DiagramType::Sequence) {
+        if (m_scene->isSequenceDiagram()) {
             switch (baseType()) {
             case WidgetBase::wt_Object:
             case WidgetBase::wt_Precondition :
@@ -1461,7 +1606,6 @@ void UMLWidget::setSelectedFlag(bool _select)
  */
 void UMLWidget::setSelected(bool _select)
 {
-    WidgetBase::setSelected(_select);
     const WidgetBase::WidgetType wt = baseType();
     if (_select) {
         if (m_scene->selectedCount() == 0) {
@@ -1482,6 +1626,8 @@ void UMLWidget::setSelected(bool _select)
             UMLApp::app()->docWindow()->updateDocumentation(true);
     }
 
+    WidgetBase::setSelected(_select);
+
     update();
 
     // selection changed, we have to make sure the copy and paste items
@@ -1958,6 +2104,7 @@ void UMLWidget::saveToXMI1(QDomDocument & qDoc, QDomElement & qElement)
       Type must be set in the child class.
     */
     WidgetBase::saveToXMI1(qDoc, qElement);
+    DiagramProxyWidget::saveToXMI1(qDoc, qElement);
     qElement.setAttribute(QLatin1String("xmi.id"), Uml::ID::toString(id()));
 
     qreal dpiScale = UMLApp::app()->document()->dpiScale();
@@ -1983,6 +2130,7 @@ bool UMLWidget::loadFromXMI1(QDomElement & qElement)
     m_nId = Uml::ID::fromString(id);
 
     WidgetBase::loadFromXMI1(qElement);
+    DiagramProxyWidget::loadFromXMI1(qElement);
     QString x = qElement.attribute(QLatin1String("x"), QLatin1String("0"));
     QString y = qElement.attribute(QLatin1String("y"), QLatin1String("0"));
     QString h = qElement.attribute(QLatin1String("height"), QLatin1String("0"));
@@ -2011,18 +2159,25 @@ bool UMLWidget::loadFromXMI1(QDomElement & qElement)
  * Adds a widget to the diagram, which is connected to the current widget
  * @param widget widget instance to add to diagram
  * @param type association type
+ * @param options widget options
  */
-void UMLWidget::addConnectedWidget(UMLWidget *widget, Uml::AssociationType::Enum type)
+void UMLWidget::addConnectedWidget(UMLWidget *widget, Uml::AssociationType::Enum type, AddWidgetOptions options)
 {
     umlScene()->addItem(widget);
     widget->setX(x() + rect().width() + 100);
     widget->setY(y());
-    widget->setSize(100, 40);
-    AssociationWidget* assoc = AssociationWidget::create(umlScene(), this, type, widget);
+    if (options & SetupSize) {
+        widget->setSize(100, 40);
+        QSizeF size = widget->minimumSize();
+        widget->setSize(size);
+    }
+    AssociationWidget* assoc = options & SwitchDirection ? AssociationWidget::create(umlScene(), widget, type, this)
+                                                    : AssociationWidget::create(umlScene(), this, type, widget);
     umlScene()->addAssociation(assoc);
-    widget->showPropertiesDialog();
-    QSizeF size = widget->minimumSize();
-    widget->setSize(size);
+    umlScene()->clearSelected();
+    umlScene()->selectWidget(widget);
+    if (options & ShowProperties)
+        widget->showPropertiesDialog();
 }
 
 /**
